-- TODO
-- Need to push UnitName back into .target, so that .target is the ID not the targetName
--   Interrupted makes the next spell cast fail when interrupted spell is
--      something like opening a chest


--embedded libs
local parser = ParserLib:GetInstance("1.1")	

HotMan = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1", "AceConsole-2.0", "AceDB-2.0", "CandyBar-2.0")
local surface = AceLibrary("Surface-1.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local paint = AceLibrary("PaintChips-2.0")

optionsTable = {
	type = "group",
	args = {
		header = {
			type = "header",
			name = "HotMan",
			order = 1,
		},
		bars = {
			type = "group",
			name = "Bar",
			desc = "Statusbar options",
			order = 5,
			args = {
				scale = {
					type = "range",
					name = "Scale",
					desc = "Change statusbar scale",
					set = function(v) HotMan.db.profile.scale = v HotMan:UpdateActiveBars() end,
					get = function() return HotMan.db.profile.scale end,
					min = 0.5, max = 2, step = 0.1,
					order = 1,
				},
				width = {
					type = "range",
					name = "Width",
					desc = "Change statusbar width",
					set = function(v) HotMan.db.profile.width = v HotMan:UpdateActiveBars() end,
					get = function() return HotMan.db.profile.width end,
					min = 50, max = 300, step = 10,
					order = 2,
				},
				height = {
					type = "range",
					name = "Height",
					desc = "Change statusbar thickness",
					set = function(v) HotMan.db.profile.height = v HotMan:UpdateActiveBars() end,
					get = function() return HotMan.db.profile.height end,
					min = 10, max = 40, step = 2,
					order = 3,
				},
				texture = {
					type = "text",
					name = "Texture",
					desc = "Statusbar textures",
					set = function(v) HotMan.db.profile.texture = v HotMan:UpdateActiveBars() end,
					get = function() return HotMan.db.profile.texture end,
					validate = surface:List(),
					order = 4,
				},
				fontsize = {
					type = "range",
					name = "Font size",
					desc = "Change statusbar font size",
					set = function(v) HotMan.db.profile.fontsize = v HotMan:UpdateActiveBars() end,
					get = function() return HotMan.db.profile.fontsize end,
					min = 8, max = 20, step = 0.2,
					order = 5,
				},
				test = {
					type = "execute",
					name = "Test",
					desc = "Start a test bar",
					func = function() HotMan:StartBar("Testing", 30, "Interface\\Icons\\Ability_ThunderBolt", "lightgrey") end,
					order = 6,
				},
			},
		},
		anchor = {
			type = "execute",
			name = "Show/Hide Anchor",
			desc = "Show/Hide the bars anchor (can also left-click a statusbar)",
			func = function() 
				if HotManFrame:IsVisible() then 
					HotManFrame:Hide()
				else 
					HotManFrame:Show()
				end 
			end,
			order = 7,
		},
	},
}

HotManHOT = {
	name = nil,
	rank = nil,
	duration = nil,
	harmful = nil,
	targeted = nil
}

HotManHOTs = {}

function boolString( bool )
	if bool then return bool else return "false" end
end

function newHOT( name, rank, duration, harmful, targeted )
	hot = {name=name, rank=rank, duration=duration, harmful=harmful, targeted=targeted}
	if rank then
		rankStr = "Rank "..rank
		if not HotManHOTs[name] then
			HotManHOTs[name] = {}
		end
		HotManHOTs[name][rankStr] = hot
	else
		HotManHOTs[name] = hot
	end
	if harmful then h = "harmful" else h = "helpful" end
	--HotMan:Print("Adding hot: "..name.."  "..duration.."  "..h)
end

function hotIsTargeted( name )
	local hot = getHOT( name, 1 )
	if hot then
		return hot.targeted
	else
		return nil
	end
end
		

function getHOT( name, rank )
	--HotMan:Print("Getting hot: "..name.."  "..rank)
	if not HotManHOTs[name] then
		return nil
	elseif not HotManHOTs[name][rank] then
		-- unranked
		return HotManHOTs[name]
	else
		return HotManHOTs[name][rank]
	end
end


HotManSelectedSpell = {
	id = nil,
	name = nil,
	rank = nil,
	target = nil,
	targetAtCast = nil,
	amount = nil,
	
	setData = function (self, id, name, rank, target, targetAtCast)
		self.id = id;
		self.name = name;
		self.rank = rank;
		self.target = target;
		self.targetAtCast = targetAtCast;
		self.amount = 0;
	end,

	clear = function(self)
		self.id = nil;
		self.name = nil;
		self.rank = nil;
		self.target = nil;
		self.targetAtCast = nil;
		self.amount = 0;
	end,
};

function HotMan:OnInitialize()
	-- chat msgs
	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS", HotMan.OnCombatEvent)
	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS", HotMan.OnCombatEvent)

	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE", HotMan.OnCombatEvent)
	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE", HotMan.OnCombatEvent)

	parser:RegisterEvent("HotMan", "CHAT_MSG_COMBAT_HOSTILE_DEATH", HotMan.OnCombatEvent)
	parser:RegisterEvent("HotMan", "CHAT_MSG_COMBAT_FRIENDLY_DEATH", HotMan.OnCombatEvent)
	
	-- Misses
	parser:RegisterEvent("HotMan", "CHAT_MSG_COMBAT_SELF_MISSES", HotMan.OnCombatEvent)
	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_CREATURE_VS_SELF_BUFF", HotMan.OnCombatEvent)
	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_SELF_DAMAGE", HotMan.OnCombatEvent)

 	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_BREAK_AURA", HotMan.OnAuraGone)
 	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_AURA_GONE_SELF", HotMan.OnAuraGone)
 	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_AURA_GONE_PARTY", HotMan.OnAuraGone)
 	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_AURA_GONE_OTHER", HotMan.OnAuraGone)
  
	-- casting
	self:RegisterEvent("UNIT_SPELLCAST_STOP", "StartHOT")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", HotMan.SpellCastFail)
	--self:RegisterEvent("UNIT_SPELLCAST_FAILED", HotMan.SpellCastFail)

	self:RegisterDB("HotManDB")
	self:RegisterDefaults("profile", {
		texture = "BantoBarReverse",
		fontsize = 10,
		width = 200,
		height = 16,
		scale = 1,
	} )

	self:BulkWork()

	self:RegisterChatCommand({ "/hotman"}, optionsTable)

-- 	self:RegisterChatCommand({ "/hotman"}, {
-- 		type = "execute", name = "HotMan", desc = "HotMan",
-- 		func = function() self:Options() end,
-- 	})


	self.EVENT_DELAY = 0.25

	self.hots = {}
	self.misses = {}
	self.next_cast_is_failure = false

	
	self:AddHots()
	
	self:SecureHook("UseAction")
	self:SecureHook("CastSpell")
	self:SecureHook("CastSpellByName")
	self:SecureHook("SpellTargetUnit")
	self:SecureHook("TargetUnit", "SpellTargetUnit")
	--self:HookReport()
	
	--self:Options()
end

--------------------------------------------------------------------------------
--
-- PARSING HEALING
--
--------------------------------------------------------------------------------
function HotMan:OnCombatEvent(info)
	local self = HotMan 
	
	-- self:Print( "Skill " .. info.type .. " - " .. boolString(info.skill) .." from ".. boolString(info.source) .." to ".. boolString(info.victim) .. " isDOT " .. boolString(info.isDOT))
	if info.type == "heal" and info.source == ParserLib_SELF then
		self:Print( "I healed " .. info.victim .." for ".. info.amount .." from ".. info.skill .." and dotP is "..boolString(info.isDOT))
		if info.isDOT then
			target = self:VictimTarget(info.victim)
			overheal = max(info.amount - (UnitHealthMax(target) - UnitHealth(target)), 0)
			self:UpdateHOT(info, info.amount - overheal)
		end
	elseif info.type == "hit" and info.isDOT and info.source == ParserLib_SELF then
		--ChatFrame1:AddMessage( "I DOTed " .. info.victim .." for ".. info.amount .." using ".. info.skill)
		self:UpdateHOT(info, info.amount)
	elseif info.type == "miss" and info.source == ParserLib_SELF then
		--ChatFrame1:AddMessage( "My "..info.skill.." missed " .. info.victim)
		self.misses[getBarName( self:VictimName(info.victim), info.skill )] = true
		--self:ClearBars( , info.skill, true )
	elseif info.type == "death" then
		self:ClearBars(self:VictimName(info.victim))
	end
end

function HotMan:VictimTarget( victim )
	if victim == ParserLib_SELF then 
		return "player" 
	else 
		return victim
	end
end

function HotMan:VictimName( victim )
	if victim == ParserLib_SELF then 
		return UnitName("player") 
	else 
		return victim
	end
end


function HotMan:OnAuraGone(info)
	local self = HotMan 
	self:ClearBars(self:VictimName(info.victim), info.skill)
end

function HotMan:UpdateHOT(info, amount)
	target = self:VictimName(info.victim)
	barName = getBarName( target, info.skill )
	--self.Print("UpdateHOT "..barName)
	if self.hots[barName] then
		-- spell is being tracked
		spell = self.hots[barName]
		spell.amount = spell.amount + amount
		local hot = getHOT(spell.name, spell.rank)
		if hot.harmful then symbol = "-" else symbol = "+" end
		self:SetCandyBarText(barName, spell.target.." "..symbol..spell.amount)
	end
end

function HotMan:SpellCastFail()
	local self = HotMan
	--self:Print("SpellCastFail")
	self.next_cast_is_failure = true
end

function HotMan:ClearBars(target, skill)
	if skill then
		-- remove a single skill
		--self:Print("Removing: "..target.."  "..skill)
		barName = getBarName( target, skill )

		if self.hots[barName] then
 			self:StopCandyBar(barName)
		end
	else
		-- Removes bars corresponding to a target
		for barName,spell in pairs(self.hots) do
			if spell.target == target then
				self:StopCandyBar(barName)
			end
		end
	end
end

--------------------------------------------------------------------------------
--
-- SETUP FUNCTIONS
--
--------------------------------------------------------------------------------
function HotMan:AddHots()
	newHOT( "Renew", nil, 15, nil, true )
	newHOT( "Shadow Word: Pain", nil, 18, true, true )

	-- Druid spells [there has to be a better way to list these!]
	newHOT( "Moonfire", nil, 12, true, true )
	newHOT( "Rejuvenation", nil, 12, nil, true )
	newHOT( "Entangling Roots", 1, 12, true, true )
	newHOT( "Entangling Roots", 2, 15, true, true )
	newHOT( "Entangling Roots", 3, 18, true, true )
	newHOT( "Entangling Roots", 4, 21, true, true )
	newHOT( "Entangling Roots", 5, 24, true, true )
	newHOT( "Entangling Roots", 6, 27, true, true )
	newHOT( "Entangling Roots", 7, 27, true, true )
	newHOT( "Regrowth", nil, 21, nil, true )
	newHOT( "Innervate", nil, 20, nil, true )
	newHOT( "Faerie Fire", nil, 40, true, true )
	newHOT( "Faerie Fire (Feral)", nil, 40, true, true )
	newHOT( "Hibernate", 1, 20, true, true )
	newHOT( "Hibernate", 2, 30, true, true )
	newHOT( "Hibernate", 3, 40, true, true )
	newHOT( "Tiger's Fury", nil, 6, nil, nil )
	newHOT( "Soothe Animal", nil, 15, true, true )
	newHOT( "Abolish Poison", nil, 8, nil, true )
	newHOT( "Insect Swarm", nil, 12, true, true )
	newHOT( "Bash", nil, 3, true, true )	-- Bear
	newHOT( "Rip", nil, 12, true, true )	-- kitty
	newHOT( "Rake", nil, 9, true, true )  -- kitty
	newHOT( "Pounce", nil, 18, true, true )  -- kitty
	newHOT( "Mangle", nil, 12, true, true )  -- kitty and bear
	newHOT( "Mangle (Cat)", nil, 12, true, true )  -- kitty
	newHOT( "Mangle (Bear)", nil, 12, true, true )  -- bear
	newHOT( "Demoralizing Roar", nil, 30, true, nil )  -- bear
	newHOT( "Enrage", nil, 10, true, nil )  -- bear
	newHOT( "Challenging Roar", nil, 6, true, nil )  -- bear
	newHOT( "Frenzied Regeneration", nil, 10, nil, nil )  -- bear
	newHOT( "Barkskin", nil, 15, nil, nil )  -- Tree?
	newHOT( "Lifebloom", nil, 7, nil, true )  -- Tree
	newHOT( "Lacerate", nil, 15, true, true )  -- Tree?
	newHOT( "Feral Charge", nil, 4, true, true )  -- Tree?
end

function HotMan:Options()
	dewdrop:Open( HotManFrame,
		'children', function() dewdrop:FeedAceOptionsTable(optionsTable) end,
		'cursorX', true, 'cursorY', true )
end

function HotMan:BulkWork()
	self.textures = { }
	for name, texture in pairs(surface:Iterate()) do
		self.textures[name] = texture
	end

	self.activebars = { }

	paint:RegisterColor("grey", 0.3, 0.3, 0.3)
	paint:RegisterColor("lightgrey", 0.6, 0.6, 0.6)
	
	self:CreateBackbone()
end

--------------------------------------------------------------------------------
--
-- ADDING TIMER BARS
--
--------------------------------------------------------------------------------

-- print chat msg
function HotMan:PrintChat(msg)
	self:Print("CHAT MSG: "..msg)
end

--function getBarName(spellInfo)
--	return spellInfo.target.." -- "..spellInfo.name
--end

function getBarName(target, name)
	if hotIsTargeted(name) then
		return target.." -- "..name
	else
		return name
	end
end

-- unit aura change
function HotMan:StartHOT()
	if HotManHOTs[HotManSelectedSpell.name] then
		local hot = getHOT(HotManSelectedSpell.name, HotManSelectedSpell.rank)
		
		if hot.harmful then
			--self:Print("StartHOT: harmful "..hot.name)
			color = "red" 
		else 
			--self:Print("StartHOT: helpful")
			color = "green" 
		end
		
		if HotManSelectedSpell.target == "targeting" then
			HotManSelectedSpell.target = HotMan:DetermineTarget(true)
		end

		if not HotManSelectedSpell.target and hotIsTargeted(HotManSelectedSpell.name) then
			-- Handle the situation where the spell really is untargeted
			-- i.e., vanish, stealth
			return
		end

		barName = getBarName(HotManSelectedSpell.target, HotManSelectedSpell.name)
		
		if hotIsTargeted(HotManSelectedSpell.name) then
			text = target
		else
			text = HotManSelectedSpell.name
		end
		
		texture = GetSpellTexture( HotManSelectedSpell.id, BOOKTYPE_SPELL )
		--self:Print("StartHOT: "..HotManSelectedSpell.name.."  rank "..HotManSelectedSpell.rank.."  target  "..HotManSelectedSpell.target)

		-- start bar and remember info about the spell
		self.hots[barName] = tablecopy(HotManSelectedSpell)
		
		self:ScheduleEvent( MyStartBar, self.EVENT_DELAY, barName, hot.duration, texture, color, HotManSelectedSpell.target, text )
		--self:StartBar(barName, hot.duration, texture, color, nil, nil, HotManSelectedSpell.target )
		HotManSelectedSpell:clear()
	end
end

function MyStartBar( barName, duration, texture, color, target, text )
	local self = HotMan

	--self:Print("MyStarBar: "..barName) 
	if self.misses[barName] or self.next_cast_is_failure then
		--self:Print("Missed previously: "..barName) 
		self.misses[barName] = nil
		self.next_cast_is_failure = false
	else
		self:StartBar(barName, duration - self.EVENT_DELAY, texture, color, nil, nil, text )
	end
end

function HotMan:DetermineTarget(ForceResolve)
	-- determine the target
	id = "BUG"
	if SpellIsTargeting() and not ForceResolve then
		--self:Print("SpellIsTargeting: "..boolString(SpellCanTargetUnit("target")))
		return "targeting"
	elseif IsActionSelfCastKeyDown() then
		id = "player"
	elseif UnitName("target") and (not ForceResolve or UnitName("target") ~= HotManSelectedSpell.targetAtCast) then
		id = "target"
	elseif UnitName("mouseover") then
		id = "mouseover"
	elseif ForceResolve then
		-- we assume the spell is going to the player, as mouseover is false 
		-- when selecting the player
		id = "player"
	else
		-- Special case where the spell is untargeted -- think Stealth for rogues
		return nil
	end
	target = UnitName(id)
	--self:Print("  Target -> "..target.." Targeting? -> "..boolString(SpellIsTargeting()).."  ForceResolve -> "..boolString(ForceResolve).."  CanTarget? -> "..boolString(SpellCanTargetUnit(id)))
	--self:Print("  Target -> "..UnitName("mouseover"))
	return target
end

function HotMan:SpellTargetUnit(unit)
	HotManSelectedSpell.target = UnitName(unit)
	--self:Print("SpellTargetUnit: "..unit.." -> "..HotManSelectedSpell.target)
end

-- lists the buffs on a unit
function HotMan:ListBuffs(unitid)
	local i = 1
	while true do
		buffName, buffRank, buffTexture, buffApps = UnitBuff(unitid, i)
		if buffName == nil then break end
		if buffRank == nil then buffRank = 0 end
		self:Print("ListBuffs: "..unitid.." "..buffName.." "..buffRank)
		i = i + 1
	end
end

--------------------------------------------------------------------------------
--
-- GUI BAR FUNCTIONS
--
--------------------------------------------------------------------------------

-- creates the statusbar anchor and registers candybar group
function HotMan:CreateBackbone()
	self.mainframe = CreateFrame("Button", "HotManFrame", UIParent)
	self.mainframe:SetHeight(16)
	self.mainframe:SetWidth(200)
	self.mainframe:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	                             tile = true, tileSize = 16,
	                             insets = { left = 0, right = 0, top = 0, bottom = 0 }
	                          })
	self.mainframe:SetBackdropColor(0.3, 0.3, 0.3, 0.5)
	self.mainframe:SetMovable(true)
	self.mainframe:EnableMouse(true)
	self.mainframe:RegisterForDrag("LeftButton")
	self.mainframe:RegisterForClicks("RightButtonUp")
	self.mainframe:SetScript("OnDragStart", function() this:StartMoving() end)
	self.mainframe:SetScript("OnDragStop", function() 
		this:StopMovingOrSizing()
		local a,b,c,d,e = this:GetPoint()
		if a == "TOPLEFT" and c == "TOPLEFT" then
			self.db.profile.x = floor(d + 0.5)
			self.db.profile.y = floor(e + 0.5)
		end
	end)
	self.mainframe:SetScript("OnClick", function()
		if arg1 == "RightButton" then
			HotMan:Options()
		end
	end)
	
	self.mainframe:Hide()
	
	self.mainframe.text = self.mainframe:CreateFontString("HotManFrameText", "OVERLAY")
	self.mainframe.text:SetFont(GameFontHighlightSmall:GetFont())
	self.mainframe.text:SetText("HotMan")
	self.mainframe.text:ClearAllPoints()
	self.mainframe.text:SetAllPoints(self.mainframe)
	
	if not self.db.profile.x then
		self.mainframe:SetPoint("CENTER", UIParent, "CENTER", 0, -200)
	else
		self.mainframe:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.db.profile.x, self.db.profile.y)
	end
	
	self:RegisterCandyBarGroup("HotManBottom")
	self:SetCandyBarGroupPoint("HotManBottom", "TOP", self.mainframe, "TOP", 0, -15)
	
	self:RegisterCandyBarGroup("HotManTop")
	self:SetCandyBarGroupPoint("HotManTop", "BOTTOM", self.mainframe, "BOTTOM", 0, 15)
	self:SetCandyBarGroupGrowth("HotManTop", true)
end

-- begin a statusbar timer
-- - name: id of bar and text of bar
-- - duration: full duration of bar
-- - icon: icon location
-- - color: paintchips color
-- - nonactive: set true if you want to continue this bar timer after zoning
-- - separate: set true to anchor to the top instead of bottom
-- - specialText: bar text if different from name
function HotMan:StartBar(name, duration, icon, color, nonactive, separate, specialText)
	local self = HotMan
	if not specialText then
		self:RegisterCandyBar(name, duration, name, icon, color)
	else
		self:RegisterCandyBar(name, duration, specialText, icon, color)
	end
	self:SetCandyBarTexture(name, self.textures[self.db.profile.texture] or self.textures.default)
	self:SetCandyBarWidth(name, self.db.profile.width)
	self:SetCandyBarHeight(name, self.db.profile.height)
	self:SetCandyBarFontSize(name, self.db.profile.fontsize)
	self:SetCandyBarScale(name, self.db.profile.scale)
	self:SetCandyBarFade(name, 3)
	self:SetCandyBarBackgroundColor(name, "Grey", 0.6)
	if separate then
		self:RegisterCandyBarWithGroup(name, "HotManTop")
	else
		self:RegisterCandyBarWithGroup(name, "HotManBottom")
	end
	self:SetCandyBarOnClick(name, function(name, button) self:OnClick(name, button) end)
	self:SetCandyBarCompletion(name, function(name, button) self:OnCompletion(name, button) end, name)

	self:StartCandyBar(name, true)
	if not nonactive then
		tinsert(self.activebars, name)
	end
end

-- updates active bars with new textures if needed
function HotMan:UpdateActiveBars()
	for i=1,getn(self.activebars),1 do
		local value = self.activebars[i]
		self:SetCandyBarTexture(value, self.textures[self.db.profile.texture] or self.textures.default)
		self:SetCandyBarWidth(value, self.db.profile.width)
		self:SetCandyBarHeight(value, self.db.profile.height)
		self:SetCandyBarFontSize(value, self.db.profile.fontsize)
		self:SetCandyBarScale(value, self.db.profile.scale)
		self:UpdateCandyBarGroup("HotManBottom")
		self:UpdateCandyBarGroup("HotManTop")
	end
end

--------------------------------------------------------------------------------
--
-- BAR EVENT HANDLING
--
--------------------------------------------------------------------------------

-- gives some user friendliness to the Candy Bars
-- - left-click to toggle anchor
-- - right-click to bring up config menu
function HotMan:OnClick(name, button)
	if button == "LeftButton" then
		if HotManFrame:IsVisible() then
			HotManFrame:Hide()
		else
			HotManFrame:Show()
		end
	elseif button == "RightButton" then
		--if IsShiftKeyDown() then
		--	self:StopBar(name)
		--else
		HotMan:Options()
		--end
	end
end

function HotMan:FormatChatTime(secs)
	local math_mod = math.fmod or math.mod
	return string.format("%d:%02d", secs / 60, math_mod(secs, 60))
end

function HotMan:OnCompletion( name )
	spell = self:RemoveSpell(name)
	--self:Print("Bar completed "..name.." -- final amount "..spell.amount)
end

function HotMan:RemoveSpell( name )
	spell = self.hots[name]
	self.hots[name] = nil
	return spell
end


--------------------------------------------------------------------------------
--
-- EVENT HANDLING
--
--------------------------------------------------------------------------------

function HotMan:CastSpell(spellId, spellbookTabNum)
	--self:Print("CastSpell"..spellId.."  "..spellbookTabNum)
	HotMan:RecordCast(spellId)
end

function HotMan:CastSpellByName(spellName, onSelf)
	--self:Print("CastSpellByName"..spellName.."  "..onSelf)
end

function HotMan:UseAction(a1)
	-- Test to see if this is a macro
	if GetActionText(a1) then 
		return ;
	end
	
	t, id, subtype = GetActionInfo(a1)
	--self:Print("  -> "..t.."  "..id.."  "..subtype)

	if t == "spell" then
		HotMan:RecordCast(id)
	end
end

function HotMan:RecordCast(SpellID)
	spellName, spellRank = GetSpellName( SpellID, BOOKTYPE_SPELL )
	--self:Print("     -> "..spellName.."  "..spellRank)
	HotManSelectedSpell:setData(SpellID, spellName, spellRank, self:DetermineTarget(false), UnitName("target"));
end

--------------------------------------------------------------------------------
--
-- UTILITIES
--
--------------------------------------------------------------------------------
function tablecopy( table )
	t = {}
	for k, v in pairs(table) do t[k] = v end
	return t
end